@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (c) 2018-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -225,10 +225,26 @@ namespace rocalution
         /** \brief Write matrix to MTX (Matrix Market Format) file */
         void WriteFileMTX(const std::string& filename) const;
         /** \brief Read matrix from CSR (ROCALUTION binary format) file */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         void ReadFileCSR(const std::string& filename);
         /** \brief Write matrix to CSR (ROCALUTION binary format) file */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         void WriteFileCSR(const std::string& filename) const;
 
+        /** \brief Read a matrix from a binary file using rocsparse I/O format */
+        ROCALUTION_EXPORT
+        void ReadFileRSIO(const std::string& filename, bool maintain_initial_format = false);
+
+        /** \brief Write a matrix to binary file using rocsparse I/O format */
+        ROCALUTION_EXPORT
+        void WriteFileRSIO(const std::string& filename) const;
+
         /** \brief Sort the matrix indices
         * \details
         * Sorts the matrix by indices.
@@ -237,6 +253,9 @@ namespace rocalution
         */
         void Sort(void);
 
+        /** \brief Extract the diagonal values of the matrix into a GlobalVector */
+        void ExtractDiagonal(GlobalVector<ValueType>* vec_diag) const;
+
         /** \brief Extract the inverse (reciprocal) diagonal values of the matrix into a
         * GlobalVector
         */
@@ -276,6 +295,39 @@ namespace rocalution
                            int64_t                  m,
                            GlobalMatrix<ValueType>* pro);
 
+        /** \brief Plain aggregation - Modification of a greedy aggregation scheme from
+        * Vanek (1996)
+        */
+        ROCALUTION_EXPORT
+        void AMGGreedyAggregate(ValueType             eps,
+                                LocalVector<bool>*    connections,
+                                LocalVector<int64_t>* aggregates,
+                                LocalVector<int64_t>* aggregate_root_nodes) const;
+
+        /** \brief Parallel aggregation - Parallel maximal independent set aggregation scheme from
+        * Bell, Dalton, & Olsen (2012)
+        */
+        ROCALUTION_EXPORT
+        void AMGPMISAggregate(ValueType             eps,
+                              LocalVector<bool>*    connections,
+                              LocalVector<int64_t>* aggregates,
+                              LocalVector<int64_t>* aggregate_root_nodes) const;
+
+        /** \brief Interpolation scheme based on smoothed aggregation from Vanek (1996) */
+        ROCALUTION_EXPORT
+        void AMGSmoothedAggregation(ValueType                   relax,
+                                    const LocalVector<bool>&    connections,
+                                    const LocalVector<int64_t>& aggregates,
+                                    const LocalVector<int64_t>& aggregate_root_nodes,
+                                    GlobalMatrix<ValueType>*    prolong,
+                                    int                         lumping_strat = 0) const;
+
+        /** \brief Aggregation-based interpolation scheme */
+        ROCALUTION_EXPORT
+        void AMGUnsmoothedAggregation(const LocalVector<int64_t>& aggregates,
+                                      const LocalVector<int64_t>& aggregate_root_nodes,
+                                      GlobalMatrix<ValueType>*    prolong) const;
+
         /** \brief Ruge Stueben coarsening */
         void RSCoarsening(float eps, LocalVector<int>* CFmap, LocalVector<bool>* S) const;
         /** \brief Parallel maximal independent set coarsening for RS AMG*/
