@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (c) 2018-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -32,6 +32,8 @@
 
 namespace rocalution
 {
+    enum _itilu0_alg : unsigned int;
+    typedef _itilu0_alg ItILU0Algorithm;
 
     template <typename ValueType>
     class BaseMatrix;
@@ -480,6 +482,14 @@ namespace rocalution
         /** \brief Perform ILU(0) factorization */
         ROCALUTION_EXPORT
         void ILU0Factorize(void);
+        /** \brief Perform Iterative ILU(0) factorization */
+        ROCALUTION_EXPORT
+        void ItILU0Factorize(ItILU0Algorithm alg,
+                             int             option,
+                             int             max_iter,
+                             double          tolerance,
+                             int*            niter,
+                             double*         history);
         /** \brief Perform LU factorization */
         ROCALUTION_EXPORT
         void LUFactorize(void);
@@ -558,6 +568,76 @@ namespace rocalution
         ROCALUTION_EXPORT
         void USolve(const LocalVector<ValueType>& in, LocalVector<ValueType>* out) const;
 
+        /** \brief Analyse the structure for Iterative solve */
+        ROCALUTION_EXPORT
+        void ItLUAnalyse(void);
+        /** \brief Delete the analysed data (see ItLUAnalyse) */
+        ROCALUTION_EXPORT
+        void ItLUAnalyseClear(void);
+        /** \brief Solve LU out = in iteratively using the Jacobi method. */
+        ROCALUTION_EXPORT
+        void ItLUSolve(int                           max_iter,
+                       double                        tolerance,
+                       bool                          use_tol,
+                       const LocalVector<ValueType>& in,
+                       LocalVector<ValueType>*       out) const;
+
+        /** \brief Analyse the structure (level-scheduling) */
+        ROCALUTION_EXPORT
+        void ItLLAnalyse(void);
+        /** \brief Delete the analysed data (see ItLLAnalyse) */
+        ROCALUTION_EXPORT
+        void ItLLAnalyseClear(void);
+        /** \brief Solve LL^T out = in iteratively using the Jacobi method. */
+        ROCALUTION_EXPORT
+        void ItLLSolve(int                           max_iter,
+                       double                        tolerance,
+                       bool                          use_tol,
+                       const LocalVector<ValueType>& in,
+                       LocalVector<ValueType>*       out) const;
+        /** \brief Solve LL^T out = in iteratively using the Jacobi method. */
+        ROCALUTION_EXPORT
+        void ItLLSolve(int                           max_iter,
+                       double                        tolerance,
+                       bool                          use_tol,
+                       const LocalVector<ValueType>& in,
+                       const LocalVector<ValueType>& inv_diag,
+                       LocalVector<ValueType>*       out) const;
+
+        /** \brief Analyse the structure (level-scheduling) L-part
+      * - diag_unit == true the diag is 1;
+      * - diag_unit == false the diag is 0;
+      */
+        ROCALUTION_EXPORT
+        void ItLAnalyse(bool diag_unit = false);
+        /** \brief Delete the analysed data (see ItLAnalyse) L-part */
+        ROCALUTION_EXPORT
+        void ItLAnalyseClear(void);
+        /** \brief Solve L out = in iteratively using the Jacobi method. */
+        ROCALUTION_EXPORT
+        void ItLSolve(int                           max_iter,
+                      double                        tolerance,
+                      bool                          use_tol,
+                      const LocalVector<ValueType>& in,
+                      LocalVector<ValueType>*       out) const;
+
+        /** \brief Analyse the structure (level-scheduling) U-part;
+      * - diag_unit == true the diag is 1;
+      * - diag_unit == false the diag is 0;
+      */
+        ROCALUTION_EXPORT
+        void ItUAnalyse(bool diag_unit = false);
+        /** \brief Delete the analysed data (see ItUAnalyse) U-part */
+        ROCALUTION_EXPORT
+        void ItUAnalyseClear(void);
+        /** \brief Solve U out = in iteratively using the Jacobi method. */
+        ROCALUTION_EXPORT
+        void ItUSolve(int                           max_iter,
+                      double                        tolerance,
+                      bool                          use_tol,
+                      const LocalVector<ValueType>& in,
+                      LocalVector<ValueType>*       out) const;
+
         /** \brief Compute Householder vector */
         ROCALUTION_EXPORT
         void Householder(int idx, ValueType& beta, LocalVector<ValueType>* vec) const;
@@ -622,9 +702,53 @@ namespace rocalution
       *   mat.ReadFileCSR("my_matrix.csr");
       * \endcode
       */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void ReadFileCSR(const std::string& filename);
 
+        /** \brief Read a matrix from a binary file using rocsparse I/O format
+        * \details
+        * Read a matrix from a binary file using rocsparse I/O format.
+        *
+        * By default, the rocalution matrix object will be converted to fit the
+        * matrix format stored in the file as follows
+        * \code{.cpp}
+        *   LocalMatrix<double> mat;
+        *
+        *   // Read from file (file contains e.g. ELL matrix)
+        *   mat.ReadFileRSIO("myellmatrix.rsio");
+        *
+        *   // Print ELL matrix info
+        *   mat.Info();
+        * \endcode
+        *
+        * In the above example, the rocalution matrix will be converted to the
+        * format, that is specified in myellmatrix.rsio (e.g. ELL).
+        * Alternatively, the user can maintain the initially specified format of
+        * mat as follows
+        * \code{.cpp}
+        *   LocalMatrix<double> mat;
+        *
+        *   // Convert the matrix to COO
+        *   mat.ConvertToCOO();
+        *
+        *   // Read form file (file contains e.g. ELL matrix)
+        *   mat.ReadFileRSIO("myellmatrix.rsio", true);
+        *
+        *   // Print COO matrix info
+        *   mat.Info();
+        * \endcode
+        *
+        * In the second example, the initial matrix format is maintained.
+        * The rocalution matrix format will stay COO, regardless of the matrix
+        * format in the supplied file.
+        */
+        ROCALUTION_EXPORT
+        void ReadFileRSIO(const std::string& filename, bool maintain_initial_format = false);
+
         /** \brief Write CSR matrix to binary file
       * \details
       * Write a CSR matrix to binary file.
@@ -664,9 +788,24 @@ namespace rocalution
       *   mat.WriteFileCSR("my_matrix.csr");
       * \endcode
       */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void WriteFileCSR(const std::string& filename) const;
 
+        /** \brief Write a matrix to binary file using rocsparse I/O format
+        * \details
+        * Write a matrix to binary file using rocsparse I/O format.
+        *
+        * The matrix is written using the current rocalution matrix format,
+        * e.g. if this->GetFormat() == ELL, the matrix will be stored in
+        * ELL format.
+        */
+        ROCALUTION_EXPORT
+        void WriteFileRSIO(const std::string& filename) const;
+
         /** \brief Move all data (i.e. move the matrix) to the accelerator */
         ROCALUTION_EXPORT
         virtual void MoveToAccelerator(void);
@@ -990,22 +1129,38 @@ namespace rocalution
         void ExtractRowVector(int idx, LocalVector<ValueType>* vec) const;
 
         /** \brief Strong couplings for aggregation-based AMG */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void AMGConnect(ValueType eps, LocalVector<int>* connections) const;
         /** \brief Plain aggregation - Modification of a greedy aggregation scheme from
       * Vanek (1996)
       */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void AMGAggregate(const LocalVector<int>& connections, LocalVector<int>* aggregates) const;
 
         /** \brief Parallel aggregation - Parallel maximal independent set aggregation scheme from
       * Bell, Dalton, & Olsen (2012)
       */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void AMGPMISAggregate(const LocalVector<int>& connections,
                               LocalVector<int>*       aggregates) const;
 
         /** \brief Interpolation scheme based on smoothed aggregation from Vanek (1996) */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void AMGSmoothedAggregation(ValueType               relax,
                                     const LocalVector<int>& aggregates,
@@ -1013,10 +1168,46 @@ namespace rocalution
                                     LocalMatrix<ValueType>* prolong,
                                     int                     lumping_strat = 0) const;
         /** \brief Aggregation-based interpolation scheme */
+#if defined(WIN32) || defined(_WIN32) || defined(__WIN32)
+#else
+        [[deprecated("This function will be removed in a future release.")]]
+#endif
         ROCALUTION_EXPORT
         void AMGAggregation(const LocalVector<int>& aggregates,
                             LocalMatrix<ValueType>* prolong) const;
 
+        /** \brief Plain aggregation - Modification of a greedy aggregation scheme from
+      * Vanek (1996)
+      */
+        ROCALUTION_EXPORT
+        void AMGGreedyAggregate(ValueType             eps,
+                                LocalVector<bool>*    connections,
+                                LocalVector<int64_t>* aggregates,
+                                LocalVector<int64_t>* aggregate_root_nodes) const;
+
+        /** \brief Parallel aggregation - Parallel maximal independent set aggregation scheme from
+      * Bell, Dalton, & Olsen (2012)
+      */
+        ROCALUTION_EXPORT
+        void AMGPMISAggregate(ValueType             eps,
+                              LocalVector<bool>*    connections,
+                              LocalVector<int64_t>* aggregates,
+                              LocalVector<int64_t>* aggregate_root_nodes) const;
+
+        /** \brief Interpolation scheme based on smoothed aggregation from Vanek (1996) */
+        ROCALUTION_EXPORT
+        void AMGSmoothedAggregation(ValueType                   relax,
+                                    const LocalVector<bool>&    connections,
+                                    const LocalVector<int64_t>& aggregates,
+                                    const LocalVector<int64_t>& aggregate_root_nodes,
+                                    LocalMatrix<ValueType>*     prolong,
+                                    int                         lumping_strat = 0) const;
+        /** \brief Aggregation-based interpolation scheme */
+        ROCALUTION_EXPORT
+        void AMGUnsmoothedAggregation(const LocalVector<int64_t>& aggregates,
+                                      const LocalVector<int64_t>& aggregate_root_nodes,
+                                      LocalMatrix<ValueType>*     prolong) const;
+
         /** \brief Ruge Stueben coarsening */
         ROCALUTION_EXPORT
         void RSCoarsening(float eps, LocalVector<int>* CFmap, LocalVector<bool>* S) const;
@@ -1037,6 +1228,38 @@ namespace rocalution
                                   bool                     FF1,
                                   LocalMatrix<ValueType>*  prolong) const;
 
+        /** \brief Ruge Stueben Prolongation matrix non-zeros */
+        void RSExtPIProlongNnz(int64_t                       global_column_begin,
+                               int64_t                       global_column_end,
+                               bool                          FF1,
+                               const LocalVector<int64_t>&   l2g,
+                               const LocalVector<int>&       CFmap,
+                               const LocalVector<bool>&      S,
+                               const LocalMatrix<ValueType>& ghost,
+                               const LocalVector<PtrType>&   bnd_csr_row_ptr,
+                               const LocalVector<int64_t>&   bnd_csr_col_ind,
+                               LocalVector<int>*             f2c,
+                               LocalMatrix<ValueType>*       prolong_int,
+                               LocalMatrix<ValueType>*       prolong_gst) const;
+
+        /** \brief Ruge Stueben fill Prolongation matrix non-zeros */
+        void RSExtPIProlongFill(int64_t                       global_column_begin,
+                                int64_t                       global_column_end,
+                                bool                          FF1,
+                                const LocalVector<int64_t>&   l2g,
+                                const LocalVector<int>&       f2c,
+                                const LocalVector<int>&       CFmap,
+                                const LocalVector<bool>&      S,
+                                const LocalMatrix<ValueType>& ghost,
+                                const LocalVector<PtrType>&   bnd_csr_row_ptr,
+                                const LocalVector<int64_t>&   bnd_csr_col_ind,
+                                const LocalVector<PtrType>&   ext_csr_row_ptr,
+                                const LocalVector<int64_t>&   ext_csr_col_ind,
+                                const LocalVector<ValueType>& ext_csr_val,
+                                LocalMatrix<ValueType>*       prolong_int,
+                                LocalMatrix<ValueType>*       prolong_gst,
+                                LocalVector<int64_t>*         global_ghost_col) const;
+
         /** \brief Factorized Sparse Approximate Inverse assembly for given system matrix
       * power pattern or external sparsity pattern
       */
@@ -1095,6 +1318,13 @@ namespace rocalution
                              const int*              rG,
                              int                     rGsize) const;
 
+        /** \brief Merge ghost and Ext matrix */
+        ROCALUTION_EXPORT
+        void CompressAdd(const LocalVector<int64_t>&   l2g,
+                         const LocalVector<int64_t>&   global_ghost_col,
+                         const LocalMatrix<ValueType>& ext,
+                         LocalVector<int64_t>*         global_col);
+
     protected:
         /** \brief Return true if the object is on the host */
         virtual bool is_host_(void) const;
